{***************************************************************************}
{                            UNIDAD DE CONVERSION

                                  Copyright (c) 1998 Marco Antonio Alvarado
}
{***************************************************************************}

{$Q-}
UNIT UConvers;

INTERFACE

  FUNCTION ValorCifra (Cifra : Char) : Byte;
  FUNCTION ValorPosicional (Posicion, Base : Byte) : Word;
  FUNCTION ValorFrase (Frase : STRING; Base : Byte) : Word;
  FUNCTION FraseValor (Valor : Word; Base : Byte) : STRING;
  FUNCTION AjustaTamanoCeros (Frase : STRING; Ceros : Byte) : STRING;
  FUNCTION FrasePuntero (Puntero : Pointer) : STRING;

IMPLEMENTATION

  CONST
    MayorCifra = 15;
    Cifras : ARRAY [0..MayorCifra] OF Char = '0123456789ABCDEF';

  FUNCTION ValorCifra (Cifra : Char) : Byte;

    VAR
      Indice : Byte;

  BEGIN
    Cifra := UpCase (Cifra);
    Indice := MayorCifra;
    WHILE (Indice > 0) AND (Cifra <> Cifras [Indice]) DO Dec (Indice);
{Las cifras desconocidas valen 0.}
    ValorCifra := Indice;
  END;

  FUNCTION ValorPosicional (Posicion, Base : Byte) : Word;

    VAR
      Valor : Word;
      Indice : Byte;

  BEGIN
    Valor := 1;
    FOR Indice := 2 TO Posicion DO Valor := Valor * Base;
    ValorPosicional := Valor;
  END;

  FUNCTION ValorFrase (Frase : STRING; Base : Byte) : Word;

    VAR
      Valor : Word;
      Indice : Byte;
      Tamano : Byte ABSOLUTE Frase;

  BEGIN
    Valor := 0;
    FOR Indice := 1 TO Tamano DO Inc (Valor, ValorCifra (Frase [Indice]) *
      ValorPosicional (Tamano - Indice + 1, Base));
    ValorFrase := Valor;
  END;

  FUNCTION FraseValor (Valor : Word; Base : Byte) : STRING;

    VAR
      Frase : STRING;

  BEGIN

    IF Valor = 0 THEN Frase := Cifras [0] ELSE
    BEGIN
      Frase := '';

      WHILE Valor > 0 DO
      BEGIN
        Frase := Cifras [Valor MOD Base] + Frase;
        Valor := Valor DIV Base;
      END;

    END;

    FraseValor := Frase;
  END;

  FUNCTION AjustaTamanoCeros (Frase : STRING; Ceros : Byte) : STRING;

    VAR
      Tamano : Byte ABSOLUTE Frase;

  BEGIN

    IF Ceros > Tamano THEN
    BEGIN
      Dec (Ceros, Tamano);
      FOR Ceros := Ceros DOWNTO 1 DO Frase := '0' + Frase;
    END;

    AjustaTamanoCeros := Frase;
  END;

  FUNCTION FrasePuntero (Puntero : Pointer) : STRING;
  BEGIN
    FrasePuntero := FraseValor (Seg (Puntero^), 16) + ':' +
      FraseValor (Ofs (Puntero^), 16);
  END;

END.